/*
 * file CqReport.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqReport
 *
 * (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.io.OutputStream;
import java.util.List;

import javax.wvcm.Feedback;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.cq.CqQuery.FilterLeaf;
import com.ibm.rational.wvcm.stpex.StpExBase;

/**
 * A proxy interface for a ClearQuest report definition.
 * <p>
 * The user-friendly specification for the location of a report has the form
 * <pre>
 *  <b>cq.query:</b><i>&lt;parent-folder-path&gt;</i>/<i>&lt;report-name&gt;</i>@<i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 */
public interface CqReport extends CqQueryFolderItem
{
    /**
     * Whether or not this report definition is complete and ready to use for
     * generating reports.
     */
    PropertyName<Boolean> IS_VALID =
        new PropertyName<Boolean>(StpExBase.PROPERTY_NAMESPACE, "is-valid");

    /**
     * Returns the value of the {@link #IS_VALID IS_VALID} property as defined
     * by this proxy.
     * 
     * @return The value. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_VALID IS_VALID} property.
     */
    boolean getIsValid() throws WvcmException;

    /**
     * The type of record upon which this report generates reports.
     */
    PropertyName<CqRecordType> PRIMARY_RECORD_TYPE =
        new PropertyName<CqRecordType>(PROPERTY_NAMESPACE, "primary-record-type");

    /**
     * Returns the value of the {@link #PRIMARY_RECORD_TYPE PRIMARY_RECORD_TYPE}
     * property as defined by this proxy.
     * 
     * @return A CqRecordType proxy for the type of record included in reports
     *         generated by this report. May be <b>null</b> if not yet defined.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PRIMARY_RECORD_TYPE PRIMARY_RECORD_TYPE} property.
     */
    CqRecordType getPrimaryRecordType() throws WvcmException;

    /**
     * The query that selects the records to be included in the report.
     */
    PropertyName<CqQuery> QUERY =
        new PropertyName<CqQuery>(PROPERTY_NAMESPACE, "query");

    /**
     * Returns the value of the {@link #QUERY QUERY}
     * property as defined by this proxy.
     * 
     * @return A CqQuery proxy for the query that selects the records to be
     * included in the reports generated by this report definition.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #QUERY QUERY} property.
     */
    CqQuery getQuery() throws WvcmException;

    /**
     * Defines a new value for the {@link #QUERY QUERY} property of this proxy.
     * 
     * @param value A CqQuery object specifying the new query that will select
     * records for this report.
     */
    void setQuery(CqQuery value);

    /**
     * The report format that specifies how the records selected by the report's
     * query will be formatted into a report document.
     */
    PropertyName<CqReportFormat> REPORT_FORMAT =
        new PropertyName<CqReportFormat>(PROPERTY_NAMESPACE, "report-format");

    /**
     * Returns the value of the {@link #REPORT_FORMAT REPORT_FORMAT} property as
     * defined by this proxy.
     * 
     * @return A CqReportFormat proxy for the report format of this report.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #REPORT_FORMAT REPORT_FORMAT} property.
     */
    CqReportFormat getReportFormat() throws WvcmException;

    /**
     * Defines a new value for the {@link #REPORT_FORMAT REPORT_FORMAT} property
     * of this proxy.
     * 
     * @param value A CqReportFormat proxy specifying the new report format to
     *            use for this report.
     */
    void setReportFormat(CqReportFormat value);

    /**
     * Creates a new report resource at the location specified by this proxy and
     * immediately delivers it to the database.
     * 
     * @param feedback A Feedback object specifying the properties, if any, to
     *            be included in the proxy returned by this method.
     * @return A CqReport proxy for the newly created report, populated with the
     *         property values requested by the feedback argument.
     * @throws WvcmException If the report cannot be created at the specified
     *             location, one or more of the dirty properties of this
     *             resource are inappropriate for the new report, or the new
     *             report cannot be delivered to the database.
     */
    CqReport doCreateReport(Feedback feedback) throws WvcmException;

    /**
     * Creates a new report resource at the location specified by this proxy.
     * 
     * @param feedback A Feedback object specifying the properties, if any, to
     *            be included in the proxy returned by this method.
     * @param deliveryOrder If {@link CqProvider#HOLD} the created report is
     *            left in a writable state in the change context--the new report
     *            in the change context must be delivered before it becomes
     *            visible to other providers. If not {@link CqProvider#HOLD},
     *            the modified and moribund resources specified by this
     *            parameter will be delivered to or deleted from the database in
     *            the order indicated. To deliver all modified and moribund
     *            resources in an arbitrary order, use
     *            {@link CqProvider#DELIVER_ALL}. To deliver just this new
     *            report, use {@link CqProvider#DELIVER}. Must not be <b>null</b>.
     * @return A CqReport proxy for the newly created report, populated with the
     *         property values requested by the feedback argument.
     * @throws WvcmException If the report cannot be created at the specified
     *             location, one or more of the dirty properties of this
     *             resource are inappropriate for the new report, or one of the
     *             resources to be delivered cannot be delivered.
     */
    CqReport doCreateReport(Feedback feedback,
                            List<CqContextResource> deliveryOrder)
        throws WvcmException;

    /**
     * Executes the query associated with this report and generates an HTML
     * formatted report from the result set based on the report's designated
     * report format.
     * 
     * @param reportHTML An open OutputSteam that will receive the generated
     *            HTML.
     * @param targetRow The first row of the result set to be included in the
     *            report.
     * @param maxRows The maximum number of rows to be included in the report
     * @param dynamicFilters Any dynamic filters needed to execute the query.
     * @throws WvcmException If there are errors
     */
    void doMakeReport(OutputStream reportHTML,
                      long targetRow,
                      long maxRows,
                      FilterLeaf... dynamicFilters) throws WvcmException;
}
